@@ -84,6 +84,10 @@ struct CDRStltRecord
   unsigned dropCapId;
 };
 
+static void processNameForEncoding(WPXString & /* name */, unsigned short & /* encoding */)
+{
+}
+
 } // anonymous namespace
 
 libcdr::CDRParser::CDRParser(const std::vector<WPXInputStream *> &externalStreams, libcdr::CDRCollector *collector)
@@ -113,7 +117,7 @@ bool libcdr::CDRParser::parseWaldo(WPXInputStream *input)
     unsigned i = 0;
     for (i = 0; i < 8; ++i)
       offsets.push_back(readU32(input));
-    /* unsigned char flag = */ readU8(input);
+    input->seek(1, WPX_SEEK_CUR);
     for (i = 0; i < 10; i++)
       offsets.push_back(readU32(input));
     input->seek(offsets[0], WPX_SEEK_SET);
@@ -171,12 +175,13 @@ bool libcdr::CDRParser::parseWaldo(WPXInputStream *input)
         if (iter7 != records7.end())
           input->seek(iter7->second.offset, WPX_SEEK_SET);
         input->seek(0x26, WPX_SEEK_CUR);
-        trafo.m_v0 = readFixedPoint(input);
-        trafo.m_v1 = readFixedPoint(input);
-        trafo.m_x0 = readFixedPoint(input) / 1000.0;
-        trafo.m_v3 = readFixedPoint(input);
-        trafo.m_v4 = readFixedPoint(input);
-        trafo.m_y0 = readFixedPoint(input) / 1000.0;
+        double v0 = readFixedPoint(input);
+        double v1 = readFixedPoint(input);
+        double v2 = readFixedPoint(input) / 1000.0;
+        double v3 = readFixedPoint(input);
+        double v4 = readFixedPoint(input);
+        double v5 = readFixedPoint(input) / 1000.0;
+        trafo = CDRTransform(v0, v1, v2, v3, v4, v5);
       }
       records1[iterVec->id] = WaldoRecordType1(iterVec->id, next, previous, child, parent, flags, x0, y0, x1, y1, trafo);
     }
@@ -281,9 +286,9 @@ bool libcdr::CDRParser::parseWaldoStructure(WPXInputStream *input, std::stack<Wa
       {
         m_collector->collectGroup(waldoStack.size());
         m_collector->collectSpnd(waldoStack.top().m_id);
-        m_collector->collectTransform(waldoStack.top().m_trafo.m_v0, waldoStack.top().m_trafo.m_v1, waldoStack.top().m_trafo.m_x0,
-                                      waldoStack.top().m_trafo.m_v3, waldoStack.top().m_trafo.m_v4, waldoStack.top().m_trafo.m_y0,
-                                      true);
+        CDRTransforms trafos;
+        trafos.append(waldoStack.top().m_trafo);
+        m_collector->collectTransform(trafos, true);
       }
       iter1 = records1.find(waldoStack.top().m_child);
       if (iter1 == records1.end())
@@ -373,8 +378,10 @@ void libcdr::CDRParser::readWaldoTrfd(WPXInputStream *input)
     v4 = readFixedPoint(input);
     y0 += readFixedPoint(input) / 1000.0;
   }
-  CDR_DEBUG_MSG(("CDRParser::readWaldoTrfd %f %f %f %f %f %f %i\n", v0, v1, x0, v3, v4, y0, m_version));
-  m_collector->collectTransform(v0, v1, x0, v3, v4, y0, m_version < 400);
+  CDR_DEBUG_MSG(("CDRParser::readWaldoTrfd %f %f %f %f %f %f %u\n", v0, v1, x0, v3, v4, y0, m_version));
+  CDRTransforms trafos;
+  trafos.append(v0, v1, x0, v3, v4, y0);
+  m_collector->collectTransform(trafos, m_version < 400);
 }
 
 void libcdr::CDRParser::readWaldoLoda(WPXInputStream *input, unsigned length)
@@ -459,7 +466,7 @@ bool libcdr::CDRParser::parseRecord(WPXInputStream *input, unsigned *blockLength
       else
         m_collector->collectOtherList();
     }
-    CDR_DEBUG_MSG(("Record: level %u %s, length: 0x%.8x (%i)\n", level, toFourCC(fourCC), length, length));
+    CDR_DEBUG_MSG(("Record: level %u %s, length: 0x%.8x (%u)\n", level, toFourCC(fourCC), length, length));
 
     if (fourCC == FOURCC_RIFF || fourCC == FOURCC_LIST)
     {
@@ -490,7 +497,7 @@ bool libcdr::CDRParser::parseRecord(WPXInputStream *input, unsigned *blockLength
         else
           m_precision = libcdr::PRECISION_32BIT;
       }
-      else if (listType == FOURCC_vect)
+      else if (listType == FOURCC_vect || listType == FOURCC_clpt)
         m_collector->collectVect(level);
 
       bool compressed = (listType == FOURCC_cmpr ? true : false);
@@ -1161,6 +1168,7 @@ void libcdr::CDRParser::readRectangle(WPXInputStream *input)
   }
   else
     m_collector->collectLineTo(0.0, 0.0);
+  m_collector->collectClosePath();
 }
 
 void libcdr::CDRParser::readEllipse(WPXInputStream *input)
@@ -1210,6 +1218,7 @@ void libcdr::CDRParser::readEllipse(WPXInputStream *input)
     m_collector->collectMoveTo(x0, y0);
     m_collector->collectArcTo(rx, ry, false, true, x1, y1);
     m_collector->collectArcTo(rx, ry, true, true, x0, y0);
+    m_collector->collectClosePath();
   }
 }
 
@@ -1371,7 +1380,7 @@ void libcdr::CDRParser::readWaldoOutl(WPXInputStream *input)
   input->seek(7, WPX_SEEK_CUR);
   unsigned short numDash = readU8(input);
   int fixPosition = input->tell();
-  std::vector<unsigned short> dashArray;
+  std::vector<unsigned> dashArray;
   for (unsigned short i = 0; i < numDash; ++i)
     dashArray.push_back(readU8(input));
   input->seek(fixPosition + 10, WPX_SEEK_SET);
@@ -1491,6 +1500,7 @@ void libcdr::CDRParser::readTrfd(WPXInputStream *input, unsigned length)
   while (i<numOfArgs)
     argOffsets[i++] = readUnsigned(input);
 
+  CDRTransforms trafos;
   for (i=0; i < argOffsets.size(); i++)
   {
     input->seek(startPosition+argOffsets[i], WPX_SEEK_SET);
@@ -1525,7 +1535,7 @@ void libcdr::CDRParser::readTrfd(WPXInputStream *input, unsigned length)
         v4 = readFixedPoint(input);
         y0 = (double)readS32(input) / 1000.0;
       }
-      m_collector->collectTransform(v0, v1, x0, v3, v4, y0, m_version < 400);
+      trafos.append(v0, v1, x0, v3, v4, y0);
     }
     else if (tmpType == 0x10)
     {
@@ -1555,6 +1565,8 @@ void libcdr::CDRParser::readTrfd(WPXInputStream *input, unsigned length)
 #endif
     }
   }
+  if (!trafos.empty())
+    m_collector->collectTransform(trafos,m_version < 400);
   input->seek(startPosition+chunkLength, WPX_SEEK_SET);
 }
 
@@ -1613,7 +1625,7 @@ void libcdr::CDRParser::readFild(WPXInputStream *input, unsigned length)
     gradient.m_centerYOffset = readInteger(input);
     if (m_version >= 600)
       input->seek(2, WPX_SEEK_CUR);
-    gradient.m_mode = (unsigned short)(readUnsigned(input) & 0xffff);
+    gradient.m_mode = (unsigned char)(readUnsigned(input) & 0xff);
     gradient.m_midPoint = (double)readU8(input) / 100.0;
     input->seek(1, WPX_SEEK_CUR);
     unsigned short numStops = (unsigned short)(readUnsigned(input) & 0xffff);
@@ -1800,8 +1812,8 @@ void libcdr::CDRParser::readFild(WPXInputStream *input, unsigned length)
         input->seek(4, WPX_SEEK_CUR);
       rcpOffset = (double)readU16(input) / 100.0;
       flags = readU8(input);
-      patternWidth = (double)tmpWidth / (m_version < 600 ? 1000.0 :254000.0);
-      patternHeight = (double)tmpHeight / (m_version < 600 ? 1000.0 :254000.0);
+      patternWidth = (double)tmpWidth / 254000.0;
+      patternHeight = (double)tmpHeight / 254000.0;
       isRelative = false;
       if ((flags & 0x04) && (m_version < 900))
       {
@@ -1813,8 +1825,7 @@ void libcdr::CDRParser::readFild(WPXInputStream *input, unsigned length)
         input->seek(17, WPX_SEEK_CUR);
       else
         input->seek(21, WPX_SEEK_CUR);
-      if (m_version >= 600)
-        patternId = readUnsigned(input);
+      patternId = readUnsigned(input);
     }
     imageFill = libcdr::CDRImageFill(patternId, patternWidth, patternHeight, isRelative, tileOffsetX, tileOffsetY, rcpOffset, flags);
   }
@@ -1862,7 +1873,7 @@ void libcdr::CDRParser::readOutl(WPXInputStream *input, unsigned length)
     input->seek(16, WPX_SEEK_CUR);
   unsigned short numDash = readU16(input);
   int fixPosition = input->tell();
-  std::vector<unsigned short> dashArray;
+  std::vector<unsigned> dashArray;
   for (unsigned short i = 0; i < numDash; ++i)
     dashArray.push_back(readU16(input));
   if (m_version < 600)
@@ -2094,7 +2105,12 @@ void libcdr::CDRParser::readBmp(WPXInputStream *input, unsigned length)
     input->seek(2, WPX_SEEK_CUR);
     unsigned short palettesize = readU16(input);
     for (unsigned short i = 0; i <palettesize; ++i)
-      palette.push_back(readU8(input) | (readU8(input) << 8) | (readU8(input) << 16));
+    {
+      unsigned b = readU8(input);
+      unsigned g = readU8(input);
+      unsigned r = readU8(input);
+      palette.push_back(b | (g << 8) | (r << 16));
+    }
   }
   std::vector<unsigned char> bitmap(bmpsize);
   unsigned long tmpNumBytesRead = 0;
@@ -2194,7 +2210,9 @@ void libcdr::CDRParser::readFtil(WPXInputStream *input, unsigned length)
   double v3 = readDouble(input);
   double v4 = readDouble(input);
   double y0 = readDouble(input) / 254000.0;
-  m_collector->collectFillTransform(v0, v1, x0, v3, v4, y0);
+  CDRTransforms fillTrafos;
+  fillTrafos.append(v0, v1, x0, v3, v4, y0);
+  m_collector->collectFillTransform(fillTrafos);
 }
 
 void libcdr::CDRParser::readVersion(WPXInputStream *input, unsigned length)
@@ -2318,6 +2336,7 @@ void libcdr::CDRParser::readFont(WPXInputStream *input, unsigned length)
       else
         break;
     }
+    processNameForEncoding(name, fontEncoding);
   }
   m_collector->collectFont(fontId, fontEncoding, name);
 }
@@ -2340,24 +2359,18 @@ void libcdr::CDRParser::readStlt(WPXInputStream *input, unsigned length)
       return;
     unsigned numFills = readU32(input);
     CDR_DEBUG_MSG(("CDRParser::readStlt numFills 0x%x\n", numFills));
-    std::map<unsigned,unsigned> fills;
     unsigned i = 0;
     for (i=0; i<numFills; ++i)
     {
-      unsigned fillId = readU32(input);
-      input->seek(4, WPX_SEEK_CUR);
-      fills[fillId] = readU32(input);
+      input->seek(12, WPX_SEEK_CUR);
       if (m_version >= 1300)
         input->seek(48, WPX_SEEK_CUR);
     }
     unsigned numOutls = readU32(input);
     CDR_DEBUG_MSG(("CDRParser::readStlt numOutls 0x%x\n", numOutls));
-    std::map<unsigned,unsigned> outls;
     for (i=0; i<numOutls; ++i)
     {
-      unsigned outlId = readU32(input);
-      input->seek(4, WPX_SEEK_CUR);
-      outls[outlId] = readU32(input);
+      input->seek(12, WPX_SEEK_CUR);
     }
     unsigned numFonts = readU32(input);
     CDR_DEBUG_MSG(("CDRParser::readStlt numFonts 0x%x\n", numFonts));
@@ -2379,13 +2392,10 @@ void libcdr::CDRParser::readStlt(WPXInputStream *input, unsigned length)
         input->seek(20, WPX_SEEK_CUR);
     }
     unsigned numAligns = readU32(input);
-    std::map<unsigned, unsigned> aligns;
     CDR_DEBUG_MSG(("CDRParser::readStlt numAligns 0x%x\n", numAligns));
     for (i=0; i<numAligns; ++i)
     {
-      unsigned alignId = readU32(input);
-      input->seek(4, WPX_SEEK_CUR);
-      aligns[alignId] = readU32(input);
+      input->seek(12, WPX_SEEK_CUR);
     }
     unsigned numIntervals = readU32(input);
     CDR_DEBUG_MSG(("CDRParser::readStlt numIntervals 0x%x\n", numIntervals));
@@ -2469,22 +2479,6 @@ void libcdr::CDRParser::readStlt(WPXInputStream *input, unsigned length)
         CDR_DEBUG_MSG(("CDRParser::readStlt parsing styles\n"));
         unsigned num = readU32(input);
         CDR_DEBUG_MSG(("CDRParser::readStlt parsing styles num 0x%x\n", num));
-        unsigned asize = 0;
-        switch (num)
-        {
-        case 3:
-          asize = 48;
-          break;
-        case 2:
-          asize = 28;
-          break;
-        case 1:
-        default:
-          asize = 8;
-          break;
-        }
-        if (m_version <= 800 && num > 1 && !set11Flag)
-          asize -= 4;
         unsigned styleId = readU32(input);
         CDRStltRecord style;
         style.parentId = readU32(input);
@@ -2720,6 +2714,18 @@ void libcdr::CDRParser::readTxsm(WPXInputStream *input, unsigned length)
 #endif
 }
 
+void libcdr::CDRParser::_readX6StyleString(WPXInputStream *input, unsigned length, WPXString &styleString)
+{
+  std::vector<unsigned char> styleBuffer(length);
+  unsigned long numBytesRead = 0;
+  const unsigned char *tmpBuffer = input->read(length, numBytesRead);
+  if (numBytesRead)
+    memcpy(&styleBuffer[0], tmpBuffer, numBytesRead);
+  appendCharacters(styleString, styleBuffer);
+  CDR_DEBUG_MSG(("CDRParser::_readX6StyleString - styleString = \"%s\"\n", styleString.cstr()));
+}
+
+
 void libcdr::CDRParser::readTxsm16(WPXInputStream *input)
 {
 #ifndef DEBUG
@@ -2768,7 +2774,8 @@ void libcdr::CDRParser::readTxsm16(WPXInputStream *input)
     input->seek(1, WPX_SEEK_CUR);
 
     unsigned len2 = readU32(input);
-    input->seek(2*len2, WPX_SEEK_CUR);
+    WPXString styleString;
+    _readX6StyleString(input, 2*len2, styleString);
 
     unsigned numRecords = readU32(input);
 
@@ -2782,10 +2789,12 @@ void libcdr::CDRParser::readTxsm16(WPXInputStream *input)
       if (flag & 0x04)
       {
         lenN = readU32(input);
-        input->seek(2*lenN, WPX_SEEK_CUR);
+        styleString.clear();
+        _readX6StyleString(input, 2*lenN, styleString);
       }
       lenN = readU32(input);
-      input->seek(2*lenN, WPX_SEEK_CUR);
+      styleString.clear();
+      _readX6StyleString(input, 2*lenN, styleString);
     }
 
     std::map<unsigned, CDRCharacterStyle> charStyles;
@@ -2795,8 +2804,7 @@ void libcdr::CDRParser::readTxsm16(WPXInputStream *input)
     {
       charDescriptions[i] = readU64(input);
     }
-    unsigned numBytes = numChars;
-    numBytes = readU32(input);
+    unsigned numBytes = readU32(input);
     unsigned long numBytesRead = 0;
     const unsigned char *buffer = input->read(numBytes, numBytesRead);
     if (numBytesRead != numBytes)
@@ -2820,7 +2828,8 @@ void libcdr::CDRParser::readTxsm6(WPXInputStream *input)
   input->seek(0x28, WPX_SEEK_CUR);
   input->seek(48, WPX_SEEK_CUR);
   input->seek(4, WPX_SEEK_CUR);
-  /* unsigned stlId = */ readU32(input);
+  /* unsigned stlId = */
+  readU32(input);
   unsigned numSt = readU32(input);
   unsigned i = 0;
   for (; i<numSt; ++i)
@@ -2837,7 +2846,8 @@ void libcdr::CDRParser::readTxsm6(WPXInputStream *input)
 void libcdr::CDRParser::readTxsm5(WPXInputStream *input)
 {
   input->seek(10, WPX_SEEK_CUR);
-  /* unsigned stlId = */ readU16(input);
+  /* unsigned stlId = */
+  readU16(input);
   unsigned numSt = readU16(input);
   unsigned i = 0;
   for (; i<numSt; ++i)
@@ -2865,7 +2875,8 @@ void libcdr::CDRParser::readStyd(WPXInputStream *input)
   unsigned numOfArgs = readUnsigned(input);
   unsigned startOfArgs = readUnsigned(input);
   unsigned startOfArgTypes = readUnsigned(input);
-  /* unsigned parentId = */ readUnsigned(input);
+  /* unsigned parentId = */
+  readUnsigned(input);
   std::vector<unsigned> argOffsets(numOfArgs, 0);
   std::vector<unsigned> argTypes(numOfArgs, 0);
   unsigned i = 0;
