@@ -97,7 +97,7 @@ bool libcdr::CMXParser::parseRecord(WPXInputStream *input, unsigned level)
     unsigned length = readU32(input);
     long endPosition = input->tell() + length;
 
-    CDR_DEBUG_MSG(("Record: level %u %s, length: 0x%.8x (%i)\n", level, toFourCC(fourCC), length, length));
+    CDR_DEBUG_MSG(("Record: level %u %s, length: 0x%.8x (%u)\n", level, toFourCC(fourCC), length, length));
 
     if (fourCC == FOURCC_RIFF || fourCC == FOURCC_RIFX || fourCC == FOURCC_LIST)
     {
@@ -191,7 +191,7 @@ void libcdr::CMXParser::readCMXHeader(WPXInputStream *input)
     tmpString.append((char)readU8(input));
   CDR_DEBUG_MSG(("CMX Version Minor: %s\n", tmpString.cstr()));
   m_unit = readU16(input, m_bigEndian);
-  CDR_DEBUG_MSG(("CMX Base Units: %i\n", m_unit));
+  CDR_DEBUG_MSG(("CMX Base Units: %u\n", m_unit));
   m_scale = readDouble(input, m_bigEndian);
   CDR_DEBUG_MSG(("CMX Units Scale: %.9f\n", m_scale));
   input->seek(12, WPX_SEEK_CUR);
@@ -297,13 +297,13 @@ void libcdr::CMXParser::readPage(WPXInputStream *input, unsigned length)
 
 void libcdr::CMXParser::readBeginPage(WPXInputStream *input)
 {
-  unsigned char tagId = 0;
-  unsigned short tagLength = 0;
   CDRBBox box;
   CDRTransform matrix;
   unsigned flags = 0;
   if (m_precision == libcdr::PRECISION_32BIT)
   {
+    unsigned char tagId = 0;
+    unsigned short tagLength = 0;
     do
     {
       long startOffset = input->tell();
@@ -314,7 +314,7 @@ void libcdr::CMXParser::readBeginPage(WPXInputStream *input)
         break;
       }
       tagLength = readU16(input, m_bigEndian);
-      CDR_DEBUG_MSG(("  CMXParser::readBeginPage - tagId %i, tagLength %i\n", tagId, tagLength));
+      CDR_DEBUG_MSG(("  CMXParser::readBeginPage - tagId %i, tagLength %u\n", tagId, tagLength));
       switch (tagId)
       {
       case CMX_Tag_BeginPage_PageSpecification:
@@ -353,13 +353,13 @@ void libcdr::CMXParser::readBeginGroup(WPXInputStream * /* input */)
 
 void libcdr::CMXParser::readPolyCurve(WPXInputStream *input)
 {
-  unsigned char tagId = 0;
-  unsigned short tagLength = 0;
   unsigned pointNum = 0;
   std::vector<std::pair<double, double> > points;
   std::vector<unsigned char> pointTypes;
   if (m_precision == libcdr::PRECISION_32BIT)
   {
+    unsigned char tagId = 0;
+    unsigned short tagLength = 0;
     do
     {
       long startOffset = input->tell();
@@ -370,7 +370,7 @@ void libcdr::CMXParser::readPolyCurve(WPXInputStream *input)
         break;
       }
       tagLength = readU16(input, m_bigEndian);
-      CDR_DEBUG_MSG(("  CMXParser::readPolyCurve - tagId %i, tagLength %i\n", tagId, tagLength));
+      CDR_DEBUG_MSG(("  CMXParser::readPolyCurve - tagId %i, tagLength %u\n", tagId, tagLength));
       switch (tagId)
       {
       case CMX_Tag_PolyCurve_RenderingAttr:
@@ -418,8 +418,6 @@ void libcdr::CMXParser::readPolyCurve(WPXInputStream *input)
 
 void libcdr::CMXParser::readEllipse(WPXInputStream *input)
 {
-  unsigned char tagId = 0;
-  unsigned short tagLength = 0;
   double angle1 = 0.0;
   double angle2 = 0.0;
   double rotation = 0.0;
@@ -432,6 +430,8 @@ void libcdr::CMXParser::readEllipse(WPXInputStream *input)
 
   if (m_precision == libcdr::PRECISION_32BIT)
   {
+    unsigned char tagId = 0;
+    unsigned short tagLength = 0;
     do
     {
       long startOffset = input->tell();
@@ -442,7 +442,7 @@ void libcdr::CMXParser::readEllipse(WPXInputStream *input)
         break;
       }
       tagLength = readU16(input, m_bigEndian);
-      CDR_DEBUG_MSG(("  CMXParser::readEllipse - tagId %i, tagLength %i\n", tagId, tagLength));
+      CDR_DEBUG_MSG(("  CMXParser::readEllipse - tagId %i, tagLength %u\n", tagId, tagLength));
       switch (tagId)
       {
       case CMX_Tag_Ellips_RenderingAttr:
@@ -518,8 +518,6 @@ void libcdr::CMXParser::readEllipse(WPXInputStream *input)
 
 void libcdr::CMXParser::readRectangle(WPXInputStream *input)
 {
-  unsigned char tagId = 0;
-  unsigned short tagLength = 0;
   double cx = 0.0;
   double cy = 0.0;
   double width = 0.0;
@@ -528,6 +526,8 @@ void libcdr::CMXParser::readRectangle(WPXInputStream *input)
   double angle = 0.0;
   if (m_precision == libcdr::PRECISION_32BIT)
   {
+    unsigned char tagId = 0;
+    unsigned short tagLength = 0;
     do
     {
       long startOffset = input->tell();
@@ -538,7 +538,7 @@ void libcdr::CMXParser::readRectangle(WPXInputStream *input)
         break;
       }
       tagLength = readU16(input, m_bigEndian);
-      CDR_DEBUG_MSG(("  CMXParser::readRectangle - tagId %i, tagLength %i\n", tagId, tagLength));
+      CDR_DEBUG_MSG(("  CMXParser::readRectangle - tagId %i, tagLength %u\n", tagId, tagLength));
       switch (tagId)
       {
       case CMX_Tag_Rectangle_RenderingAttr:
@@ -608,13 +608,15 @@ libcdr::CDRTransform libcdr::CMXParser::readMatrix(WPXInputStream *input)
   switch (type)
   {
   case 2: // general matrix
-    matrix.m_v0 = readDouble(input, m_bigEndian);
-    matrix.m_v3 = readDouble(input, m_bigEndian);
-    matrix.m_v1 = readDouble(input, m_bigEndian);
-    matrix.m_v4 = readDouble(input, m_bigEndian);
-    matrix.m_x0 = readDouble(input, m_bigEndian);
-    matrix.m_y0 = readDouble(input, m_bigEndian);
-    return matrix;
+  {
+    double v0 = readDouble(input, m_bigEndian);
+    double v3 = readDouble(input, m_bigEndian);
+    double v1 = readDouble(input, m_bigEndian);
+    double v4 = readDouble(input, m_bigEndian);
+    double x0 = readDouble(input, m_bigEndian);
+    double y0 = readDouble(input, m_bigEndian);
+    return libcdr::CDRTransform(v0, v1, x0, v3, v4, y0);
+  }
   default: // identity matrix for the while
     return matrix;
   }
@@ -649,7 +651,7 @@ void libcdr::CMXParser::readFill(WPXInputStream *input)
           CDR_DEBUG_MSG(("    Solid fill - tagId %i\n", tagId));
         }
         tagLength = readU16(input, m_bigEndian);
-        CDR_DEBUG_MSG(("    Solid fill - tagId %i, tagLength %i\n", tagId, tagLength));
+        CDR_DEBUG_MSG(("    Solid fill - tagId %i, tagLength %u\n", tagId, tagLength));
         switch (tagId)
         {
         case CMX_Tag_RenderAttr_FillSpec_Uniform:
@@ -707,7 +709,7 @@ void libcdr::CMXParser::readRenderingAttributes(WPXInputStream *input)
           break;
         }
         tagLength = readU16(input, m_bigEndian);
-        CDR_DEBUG_MSG(("  Fill specification - tagId %i, tagLength %i\n", tagId, tagLength));
+        CDR_DEBUG_MSG(("  Fill specification - tagId %i, tagLength %u\n", tagId, tagLength));
         switch (tagId)
         {
 
@@ -735,7 +737,7 @@ void libcdr::CMXParser::readRenderingAttributes(WPXInputStream *input)
           break;
         }
         tagLength = readU16(input, m_bigEndian);
-        CDR_DEBUG_MSG(("  Outline specification - tagId %i, tagLength %i\n", tagId, tagLength));
+        CDR_DEBUG_MSG(("  Outline specification - tagId %i, tagLength %u\n", tagId, tagLength));
         switch (tagId)
         {
         case CMX_Tag_RenderAttr_OutlineSpec:
@@ -765,7 +767,7 @@ void libcdr::CMXParser::readRenderingAttributes(WPXInputStream *input)
           break;
         }
         tagLength = readU16(input, m_bigEndian);
-        CDR_DEBUG_MSG(("  Lens specification - tagId %i, tagLength %i\n", tagId, tagLength));
+        CDR_DEBUG_MSG(("  Lens specification - tagId %i, tagLength %u\n", tagId, tagLength));
         switch (tagId)
         {
         default:
@@ -790,7 +792,7 @@ void libcdr::CMXParser::readRenderingAttributes(WPXInputStream *input)
           break;
         }
         tagLength = readU16(input, m_bigEndian);
-        CDR_DEBUG_MSG(("  Canvas specification - tagId %i, tagLength %i\n", tagId, tagLength));
+        CDR_DEBUG_MSG(("  Canvas specification - tagId %i, tagLength %u\n", tagId, tagLength));
         switch (tagId)
         {
         default:
@@ -815,7 +817,7 @@ void libcdr::CMXParser::readRenderingAttributes(WPXInputStream *input)
           break;
         }
         tagLength = readU16(input, m_bigEndian);
-        CDR_DEBUG_MSG(("  Container specification - tagId %i, tagLength %i\n", tagId, tagLength));
+        CDR_DEBUG_MSG(("  Container specification - tagId %i, tagLength %u\n", tagId, tagLength));
         switch (tagId)
         {
         default:
@@ -830,10 +832,10 @@ void libcdr::CMXParser::readRenderingAttributes(WPXInputStream *input)
 
 void libcdr::CMXParser::readJumpAbsolute(WPXInputStream *input)
 {
-  unsigned char tagId = 0;
-  unsigned short tagLength = 0;
   if (m_precision == libcdr::PRECISION_32BIT)
   {
+    unsigned char tagId = 0;
+    unsigned short tagLength = 0;
     do
     {
       long endOffset = input->tell() + tagLength;
@@ -844,7 +846,7 @@ void libcdr::CMXParser::readJumpAbsolute(WPXInputStream *input)
         break;
       }
       tagLength = readU16(input, m_bigEndian);
-      CDR_DEBUG_MSG(("  CMXParser::readJumpAbsolute - tagId %i, tagLength %i\n", tagId, tagLength));
+      CDR_DEBUG_MSG(("  CMXParser::readJumpAbsolute - tagId %i, tagLength %u\n", tagId, tagLength));
       switch (tagId)
       {
       case CMX_Tag_JumpAbsolute_Offset:
